#include "../bandit/CombUCBStep.h"

#include <algorithm>
#include <math.h>
#include <numeric>

#define EPSILON 0.0000001


CombUCBStep::CombUCBStep(int numArms, int sizeSubset)
{
    this->numArms = numArms;
    this->sizeSubset = sizeSubset;
    this->numPulls = std::vector<double>(numArms, 0.0);
    this->empiricalMeans = std::vector<double>(numArms, 1.0);
    this->times = 0.0;
    this->prevBestSupArm = std::vector<size_t>(this->sizeSubset);
    std::iota(this->prevBestSupArm.begin(), this->prevBestSupArm.end(), 0);
}

CombUCBStep::~CombUCBStep() {}

void CombUCBStep::updateBestSupArm() {
    std::vector<size_t> newArmIdxs;
    std::vector<double> newUcbs;
    double func, ucb, prevMin, prevMinIdx;

    /// Calculate UCB values.
    prevMin = 10000.0;
    for (size_t i = 0; i < this->numArms; i++) {
        func = 2.0 * log(1.0 + this->times * pow(log(this->times + EPSILON), 2.0));
        ucb = this->empiricalMeans[i] + sqrt(func / (this->numPulls[i] + EPSILON));

        auto res = std::find(this->prevBestSupArm.begin(), this->prevBestSupArm.end(), i);
        if (res != this->prevBestSupArm.end()) {
            if (ucb < prevMin) {
                prevMin = ucb;
                prevMinIdx = std::distance(this->prevBestSupArm.begin(), res);
            }
        }
        else {
            newUcbs.emplace_back(ucb);
            newArmIdxs.emplace_back(i);
        }
    }

    /// The previous minimum could also be selected.
    newUcbs.emplace_back(prevMin);
    newArmIdxs.emplace_back(this->prevBestSupArm[prevMinIdx]);

    /// Obtain the top arm.
    std::vector<double>::iterator result = std::max_element(newUcbs.begin(), newUcbs.end());
    size_t maxIdx = std::distance(newUcbs.begin(), result);
    this->prevBestSupArm[prevMinIdx] = newArmIdxs[maxIdx];
}

std::vector<size_t> CombUCBStep::getBestSupArm()
{
    return this->prevBestSupArm;
}

void CombUCBStep::receiveReward(size_t arm, double reward)
{
    this->numPulls[arm]++;

    if (this->numPulls[arm] == 1.0) {
        this->empiricalMeans[arm] = reward;
    }
    else {
        this->empiricalMeans[arm] += (reward - this->empiricalMeans[arm]) / this->numPulls[arm];
    }
}

double CombUCBStep::increaseTime()
{
    return ++this->times;
}
