#include "../bandit/CombUCB.h"

#include <algorithm>
#include <math.h>
#include <numeric>

#define EPSILON 0.0000001


CombUCB::CombUCB(int numArms, int sizeSubset)
{
    this->numArms = numArms;
    this->sizeSubset = sizeSubset;
    this->numPulls = std::vector<double>(numArms, 0.0);
    this->empiricalMeans = std::vector<double>(numArms, 1.0);
    this->times = 0.0;
}

CombUCB::~CombUCB() {}

std::vector<size_t> CombUCB::getBestSupArm()
{
    std::vector<size_t> bestSupArm;
    std::vector<size_t> bestArmIndices(this->numArms);
    std::vector<double> ucbs;
    double func;

    /// Calculate UCB values.
    for (size_t i = 0; i < this->numArms; i++) {
        func = 2.0 * log(1.0 + this->times * pow(log(this->times + EPSILON), 2.0));
        ucbs.emplace_back(this->empiricalMeans[i] + sqrt(func / (this->numPulls[i] + EPSILON)));
    }

    /// Obtain the top-k arms.
    std::iota(bestArmIndices.begin(), bestArmIndices.end(), 0);
    std::partial_sort(
        bestArmIndices.begin(), bestArmIndices.begin() + this->sizeSubset, bestArmIndices.end(),
        [&ucbs](size_t i1, size_t i2) {
            return ucbs[i1] > ucbs[i2];
        }
    );
    bestSupArm = std::vector<size_t>(bestArmIndices.begin(), bestArmIndices.begin() + this->sizeSubset);

    return bestSupArm;
}

void CombUCB::receiveReward(size_t arm, double reward)
{
    this->numPulls[arm]++;

    if (this->numPulls[arm] == 1.0) {
        this->empiricalMeans[arm] = reward;
    }
    else {
        this->empiricalMeans[arm] += (reward - this->empiricalMeans[arm]) / this->numPulls[arm];
    }
}

double CombUCB::increaseTime()
{
    return ++this->times;
}
